# -*- coding: utf-8 -*-
# Copyright (c) 2023 mashu3
# This software is released under the MIT License, see LICENSE.

import io
import os
import re
import sys
import img2pdf
import pikepdf
import tempfile
import rarfile
import zipfile
import argparse
import warnings
import numpy as np
from PIL import Image
import concurrent.futures
from bs4 import BeautifulSoup

warnings.filterwarnings('ignore', category=UserWarning)
from bs4 import XMLParsedAsHTMLWarning
warnings.filterwarnings('ignore', category=XMLParsedAsHTMLWarning)

class MangaPdfConverter():   
    def __init__(self, input_path: str, output_path: str, pagelayout:str, direction:str):
        self.input_path = input_path
        self.output_path = output_path
        self.pagelayout = pagelayout
        self.direction = direction
        self.convert_to_grayscale = False
        self.convert_to_jpeg = False
    def set_convert_to_jpeg(self, flag):
        self.convert_to_jpeg = flag
    def set_convert_to_grayscale(self, flag):
        self.convert_to_grayscale = flag

    # Function to determine w   hether the given file name is an image file or not
    def is_image_file(self, filename):
        return any(filename.lower().endswith(ext) for ext in ['.jpg', '.jpeg', '.png', '.gif', '.bmp'])
    
    # Function to determine whether the given path is an epub file or not
    def is_epub_file(self, path):
        ext = os.path.splitext(path)[1].lower()
        return ext in ['.epub']
    
    # Function to determine whether the given path is an archive file or not
    def is_archive_file(self, path):
        ext = os.path.splitext(path)[1].lower()
        return ext in ['.zip', '.cbz', '.rar', '.cbr']
    
    # Function to generate sort keys
    def sort_key(self, filename):
        filename = filename.lower()
        key = []
        if 'cover' in filename:
            key.append(True)
        else:
            key.append(False)
        for s in re.split(r'(\d+)', filename):
            if s.isdigit():
                key.append(int(s))
            else:
                key.append(s)
        if 'copyright' in filename:
            key.append(True)
        else:
            key.append(False)
        return tuple(key)
    
    # Function that returns a list of paths to image files in the specified directory
    def find_image_files(self, input_path, tmp_dir):
        img_files = []
        # If the input_path is a directory
        if os.path.isdir(input_path):
            for root, _, files in os.walk(input_path):
                for file in files:
                    if self.is_image_file(file):
                        img_files.append(os.path.join(root, file))
        # If the input_path is a zip, cbz, rar, or cbr file
        elif self.is_archive_file(input_path):
            ext = os.path.splitext(input_path)[1].lower()
            if ext in ['.zip', '.cbz']:
                if zipfile.is_zipfile(input_path):
                    with zipfile.ZipFile(input_path) as archive:
                        archive.extractall(tmp_dir)
            elif ext in ['.rar', '.cbr']:
                if rarfile.is_rarfile(input_path):
                    with rarfile.RarFile(input_path) as archive:
                        archive.extractall(tmp_dir)
            for root, _, files in os.walk(tmp_dir):
                for file in files:
                    if self.is_image_file(file):
                        img_files.append(os.path.join(root, file))
        # If the input_path is not a directory or archive file
        else:
            raise ValueError(f'{input_path} is not a directory or an archive file.')
        # Sort the list of image file paths by filename
        img_files.sort(key=self.sort_key)
        return img_files
    
    # Function to convert an image file to JPEG format and save it in a temporary directory
    def to_jpeg(self, img_file_path, tmp_dir):
        img_output_path = os.path.join(tmp_dir, os.path.basename(img_file_path)[:-4] + '.jpg')
        with Image.open(img_file_path) as im:
            im.convert('RGB').save(img_output_path, 'JPEG')
        return img_output_path, img_file_path
    
    # Function to determine whether an image is a color image or not.
    def is_color(self, img):
        # Return False if the image is grayscale.
        if img.mode == 'L':
            return False
        # Get the values of each channel in RGB.
        img = img.convert('RGB')
        r_arr = np.array(img)[:, :, 0]/ 255
        g_arr = np.array(img)[:, :, 1]/ 255
        b_arr = np.array(img)[:, :, 2]/ 255
        # Calculate the difference between each RGB channel.
        diff_rg = np.abs(r_arr - g_arr)
        diff_gb = np.abs(g_arr - b_arr)
        diff_rb = np.abs(r_arr - b_arr)

        # Get the location of each pixel where the difference between the RGB channels exceeds the threshold.
        threshold = 0.5
        not_gray_indices = np.argwhere(
            (diff_rg > threshold) | (diff_gb > threshold) | (diff_rb > threshold)
        )
        return not_gray_indices.shape[0] != 0
    
    # Function to convert PNG images to grayscale if the input image is not already grayscale.
    def to_grayscale(self, img_file_path, tmp_dir):
        img_output_path = os.path.join(tmp_dir, os.path.basename(img_file_path)[:-4] + '.png')
        with Image.open(img_file_path) as img:
            if not self.is_color(img): # If the PNG image is in black and white, perform grayscale conversion.
                img = img.convert('L')
            else:
                img = img.convert('RGB')
            img.save(img_output_path, 'PNG')
        return img_output_path, img_file_path
    
    # Function to remove alpha channel from PNG images if the input image contains an alpha channel.
    def remove_alpha_channel(self, img_file_path, tmp_dir):
        img_output_path = os.path.join(tmp_dir, os.path.basename(img_file_path)[:-4] + '.png')
        with Image.open(img_file_path) as img:
            if img.mode in ['RGBA', 'LA'] or (img.mode == 'P' and 'transparency' in img.info):
                img = img.convert('RGB')
            img.save(img_output_path, 'PNG')
        return img_output_path, img_file_path
        
    # Function to extract the contents of an EPUB file
    def extract_epub_contents(self, epub):
        contents = epub.namelist()
        for item in contents:
            extension = item.split('.')[-1].lower()
            if extension == 'ncx':
                ncx_name = item
            if extension == 'opf':
                opf_name = item
        page_names = []
        with epub.open(opf_name) as opf:
            content_opf = opf.read().decode()
            opf_soup = BeautifulSoup(content_opf, 'xml')
            for item in opf_soup.find_all('item', {'media-type': 'image/jpeg'}):
                page_names.append(os.path.join(os.path.dirname(opf_name), item.get('href').replace('/', os.sep)).replace(os.sep, '/'))
            for item in opf_soup.find_all('item', {'media-type': 'image/png'}):
                page_names.append(os.path.join(os.path.dirname(opf_name), item.get('href').replace('/', os.sep)).replace(os.sep, '/'))
        page_items = []
        for page_name in page_names:
            page_items.append(epub.open(page_name))
        return page_names, page_items, ncx_name, opf_name
    
    # Function to extract the index of an EPUB file
    def extract_epub_index(self, epub, page_names, ncx_name: str):
        page_index = []
        with epub.open(ncx_name) as ncx_file:
            ncx_content = ncx_file.read()
        ncx_soup = BeautifulSoup(ncx_content, 'lxml')
        navpoints = ncx_soup.find_all('navpoint')
        ncx_path = os.path.dirname(ncx_name) + '/'
        for navpoint in navpoints:
            nav_label = navpoint.navlabel.text.strip()
            nav_text = navpoint.content['src']
            if nav_text.startswith(ncx_path):
                nav_text = nav_text[len(ncx_path):]
            else:
                nav_text = os.path.join(ncx_path, nav_text)
            if nav_text.endswith('.xhtml'):
                with epub.open(nav_text) as xhtml_file:
                    xhtml_content = xhtml_file.read()
                xhtml_soup = BeautifulSoup(xhtml_content, 'html.parser')
                img_tags = xhtml_soup.find_all('image')
                if len(img_tags) == 0:
                    img_tags = xhtml_soup.find_all('img')
                for img_tag in img_tags:
                    try:
                        img_link = img_tag['xlink:href']
                    except KeyError:
                        img_link = img_tag['src']
                    image_href = os.path.abspath(os.path.join(os.path.dirname(nav_text), img_link))
                    image_href = os.path.relpath(image_href, os.getcwd()).replace(os.sep, '/')
                    index_number = page_names.index(image_href)
                    page_index.append([nav_label, index_number])
            else:
                index_number = page_names.index(nav_text)
                page_index.append([nav_label, index_number])
        return page_index
    
    # Function to extract the metadata of an EPUB file
    def extract_epub_metadata(self, epub, opf_name: str):
        with epub.open(opf_name) as opf_file:
            opf_content = opf_file.read()
        opf_soup = BeautifulSoup(opf_content, 'lxml')
        metadata = opf_soup.find('metadata')    
        epub_metadata = {}
        for key in ['title', 'creator', 'publisher', 'date', 'language']:
            values = metadata.find_all('dc:'+key)
            if key == 'creator':
                if values:
                    epub_metadata[key] = [value for value in values]
            else:
                if values:
                    epub_metadata[key] = values[0].text
                else:
                    epub_metadata[key] = None
        return epub_metadata
    
    # Function to convert input files to a PDF file
    def convert(self):
        if self.is_epub_file(self.input_path):
            with zipfile.ZipFile(self.input_path) as epub:
                page_names = self.extract_epub_contents(epub)[0]
                page_items = self.extract_epub_contents(epub)[1]
                ncx_name = self.extract_epub_contents(epub)[2]
                opf_name = self.extract_epub_contents(epub)[3]
                page_index = self.extract_epub_index(epub, page_names, ncx_name)
                epub_metadata = self.extract_epub_metadata(epub, opf_name)
        else:
            page_items = []
            with tempfile.TemporaryDirectory() as tmp_dir:
                img_files = self.find_image_files(self.input_path, tmp_dir)
                with concurrent.futures.ThreadPoolExecutor() as executor:
                    futures = []
                    for img_file_path in img_files:
                        if img_file_path.lower().endswith(('.jpg', '.jpeg')):
                            with open(img_file_path, 'rb') as f:
                                page_items.append((f.read(), img_file_path))
                        else:
                            if self.convert_to_jpeg:
                                future = executor.submit(self.to_jpeg, img_file_path, tmp_dir)
                                futures.append(future)
                            elif self.convert_to_grayscale:
                                future = executor.submit(self.to_grayscale, img_file_path, tmp_dir)
                                futures.append(future)
                            else:
                                future = executor.submit(self.remove_alpha_channel, img_file_path, tmp_dir)
                                futures.append(future)
                    for future in concurrent.futures.as_completed(futures):
                        img_output_path, img_file_path = future.result()
                        with open(img_output_path, 'rb') as f:
                            page_items.append((f.read(), img_file_path))
            # Reorder images so that the order of images is the same as the list of img_files
            page_items.sort(key=lambda x: img_files.index(x[1]))
            page_items = [data[0] for data in page_items]
        
        pdf_obj = io.BytesIO(img2pdf.convert(page_items))
        
        with pikepdf.Pdf.open(pdf_obj) as pdf:
            if self.is_epub_file(self.input_path):
                with pdf.open_metadata(set_pikepdf_as_editor=False) as pdf_metadata:
                    pdf_metadata['dc:title'] = epub_metadata['title'] if epub_metadata['title'] else ''
                    pdf_metadata['dc:creator'] = epub_metadata['creator'] if epub_metadata['creator'] else ''
                    pdf_metadata['dc:publisher'] = epub_metadata['publisher'] if epub_metadata['publisher'] else ''
                    pdf_metadata['xmp:CreateDate'] = epub_metadata['date'] if epub_metadata['date'] else ''
                    pdf_metadata['pdf:Language'] = epub_metadata['language'] if epub_metadata['language'] else ''
                    pdf_metadata['pdf:Producer'] = ''
                pdf_index = []
                for index in page_index:
                    pdf_index.append(pikepdf.OutlineItem(index[0], index[1]))
                with pdf.open_outline() as outline:
                    outline.root.extend(pdf_index)
            if self.pagelayout is not None:
                if not hasattr(pdf.Root, 'PageLayout') \
                or pdf.Root.PageLayout != '/' + self.pagelayout:
                    pdf.Root.PageLayout = pikepdf.Name('/' + self.pagelayout)
            if self.direction is not None:
                if not hasattr(pdf.Root, 'ViewerPreferences'):
                    pdf.Root.ViewerPreferences = pikepdf.Dictionary()
                if not hasattr(pdf.Root.ViewerPreferences, 'Direction') \
                    or pdf.Root.ViewerPreferences.Direction != '/' + self.direction:
                        pdf.Root.ViewerPreferences.Direction = pikepdf.Name('/' + self.direction)
            if self.output_path is None:
                if os.path.isdir(self.input_path):
                    pdf_filename = os.path.basename(self.input_path) + '.pdf'
                    output_path = os.path.join(self.input_path, pdf_filename).replace(os.sep, '/')
                else:
                    pdf_filename, _ = os.path.splitext(self.input_path)
                    output_path = f"{pdf_filename}.pdf"
            else:
                output_path = self.output_path
            if os.path.exists(output_path):
                os.remove(output_path)
            pdf.save(output_path, linearize=True)
        return None

class HelpFormatter(argparse.HelpFormatter):
    def __init__(self, prog, indent_increment=2, max_help_position=6, width=None):
        super().__init__(prog, indent_increment, max_help_position, width)
    def _split_lines(self, text, _):
        return text.splitlines()

def main():
    parser = argparse.ArgumentParser(description='This program converts manga/comic files(zip, epub, etc.) or directory containing image files (jpg, png, etc.) to PDF', formatter_class=HelpFormatter)
    parser.add_argument('input_path', nargs='?', metavar='input_path', type=str,
                        help='input file path or directory path')
    parser.add_argument('-o', '--output', dest='output_path', type=str, default=None,
                        help='''\
path to the output PDF file. 
If not specified, the output file name is generated from the input file or directory name.''')
    parser.add_argument('-p', '--pagelayout', type=str, default='TwoPageRight', 
                        choices=['SinglePage', 'OneColumn', 'TwoColumnLeft', 'TwoColumnRight', 'TwoPageLeft', 'TwoPageRight'],
                        help='''\
SinglePage -> Single page display
OneColumn -> Enable scrolling
TwoPageLeft -> Spread view
TwoColumnLeft -> Spread view with scrolling
(default) TwoPageRight -> Separate Cover, Spread View
TwoColumnRight -> Separate Cover, Scrolling Spread View''')
    parser.add_argument('-d', '--direction', type=str, default='R2L', choices=['L2R', 'R2L'],
                        help='''\
L2R -> Left Binding
(default)R2L -> Right Binding''')
    parser.add_argument('-j', '--jpeg', action='store_true', help='Convert images to JPEG')
    parser.add_argument('-g', '--grayscale', action='store_true', help='Convert images to grayscale')
    parser.add_argument('-gui', action='store_true', help='Launch GUI')

    args = parser.parse_args()
    if args.gui:
        import manga2pdf_gui
        manga2pdf_gui.launch_gui()
    else:
        if args.input_path is None:
            parser.print_usage()
            parser.print_help()
            sys.exit(1)
        if not os.path.isdir(args.input_path):
            ext = os.path.splitext(args.input_path)[1].lower()
            if not ext in ['.zip', '.cbz', '.rar', '.cbr', '.epub']:
                print('Error: The input file format is not supported. The currently supported formats are: .zip, .cbz, .rar, .cbr, and .epub.')
                sys.exit(1)
        if args.output_path is not None:
            if not args.output_path.endswith('.pdf'):
                print('Error: The output file must be an PDF file.')
                sys.exit(1)
        if args.grayscale and args.jpeg:
            print('Error: Cannot specify both --grayscale and --jpeg options.')
            sys.exit(1)
        
        converter = MangaPdfConverter(args.input_path, args.output_path, args.pagelayout, args.direction)
        if args.jpeg:
            converter.set_convert_to_jpeg(True)
        elif args.grayscale:
            converter.set_convert_to_grayscale(True)
        converter.convert()

if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*-
# Copyright (c) 2023 mashu3
# This software is released under the MIT License, see LICENSE.

import os
import sys
import tkinter as tk
from tkinter import ttk
from tkinter import filedialog
from tkinter import messagebox
from manga2pdf import MangaPdfConverter

class MangaPdfConverterGUI:
    def __init__(self, master):
        self.master = master
        self.language = "en" # Set default language to english
        master.title("Manga PDF Converter")
        self.master.resizable(0, 0)
        self.processing_window = None

        # Create five frames
        path_frame =ttk.Frame(master)
        conversion_frame = ttk.Frame(master)
        pagelayout_frame = ttk.Frame(master)
        direction_frame = ttk.Frame(master)
        button_frame = ttk.Frame(master)

        # Set grid layout
        path_frame.grid(row=0, column=0, columnspan=3, sticky='we')
        conversion_frame.grid(row=1, column=0, sticky='nswe')
        pagelayout_frame.grid(row=1, column=1, sticky='nswe')
        direction_frame.grid(row=1, column=2, sticky='nswe')
        button_frame.grid(row=2, column=0, columnspan=3, sticky='we')

        # Add input path label and entry
        input_frame = ttk.Frame(path_frame)
        input_frame.grid(row=0, column=0, columnspan=3, padx=5, pady=5, sticky="ew")
        self.input_label_text = {"en": "Input Path:", "ja": "入力パス:"}
        self.input_label = ttk.Label(input_frame, text=self.input_label_text[self.language])
        self.input_label.pack(side="left", padx=10, pady=5)
        self.input_file_button_text = {"en": "Select File", "ja": "ファイル選択"}
        self.input_file_button = ttk.Button(input_frame, text=self.input_file_button_text[self.language], command=self.browse_input_file)
        self.input_file_button.pack(side="right", pady=5)
        self.input_directory_button_text = {"en": "Select Directory", "ja": "フォルダ選択"}
        self.input_directory_button = ttk.Button(input_frame, text=self.input_directory_button_text[self.language], command=self.browse_input_directory)
        self.input_directory_button.pack(side="right", padx=10, pady=5)
        self.input_entry = ttk.Entry(path_frame, width=65)
        self.input_entry.grid(row=1, column=0, columnspan=3, padx=10, pady=5, sticky="ew")

        # Add output path label and entry
        output_frame = tk.Frame(path_frame)
        output_frame.grid(row=2, column=0, columnspan=3, padx=5, pady=5, sticky="ew")
        self.output_label_text = {"en": "Output Path:", "ja": "出力パス:"}
        self.output_label = ttk.Label(output_frame, text=self.output_label_text[self.language])
        self.output_label.pack(side="left", padx=10, pady=5)
        self.output_browse_button_text = {"en": "Browse", "ja": "参照"}
        self.output_browse_button = ttk.Button(output_frame, text=self.output_browse_button_text[self.language], command=self.browse_output_path)
        self.output_browse_button.pack(side="right", pady=5)
        self.auto_output_button_text = {"en": "Auto", "ja": "自動設定"}
        self.auto_output_button = ttk.Button(output_frame, text=self.auto_output_button_text[self.language], command=self.auto_output_path)
        self.auto_output_button.pack(side="right", padx=10, pady=5)
        self.output_entry = ttk.Entry(path_frame, width=65)
        self.output_entry.grid(row=3, column=0, columnspan=3, padx=10, pady=5, sticky="ew")

        # Create a LabelFrame for the conversion options
        self.conversion_label_text = {"en": "Conversion Options:", "ja": "圧縮方式:"}
        self.conversion_labelframe = ttk.LabelFrame(conversion_frame, text=self.conversion_label_text[self.language], padding=5)
        self.conversion_labelframe.grid(row=0, column=0, rowspan=10, sticky="nsew", padx=5, pady=1)

        self.conversion_var = tk.StringVar(value="none")
        self.conversion_text = {
            "en": ["No Compression", "Convert images to JPEG", "Convert images to grayscale"],
            "ja": ["圧縮なし", "JPEG画像に変換", "グレースケール画像に変換"],
        }
        self.conversion_value = ["none", "jpeg", "grayscale"]
        self.conversion_radio = []
        for i, conversion in enumerate(self.conversion_text[self.language]):
            conversion_radio = ttk.Radiobutton(
                self.conversion_labelframe, text=conversion, variable=self.conversion_var, value=self.conversion_value[i]
            )
            self.conversion_radio.append(conversion_radio)
            conversion_radio.grid(row=i+1, column=0, sticky="nsew", padx=5, pady=1)
        
        # Create a LabelFrame for the page layout options
        self.pagelayout_label_text = {"en": "Page Layout:", "ja": "ページレイアウト:"}
        self.pagelayout_labelframe = ttk.LabelFrame(pagelayout_frame, text=self.pagelayout_label_text[self.language], padding=5)
        self.pagelayout_labelframe.grid(row=0, column=1, rowspan=10, sticky="nsew", padx=5, pady=1)

        self.pagelayout_var = tk.StringVar(value="TwoPageRight")
        self.pagelayout_text = {
            "en": ["SinglePage", "OneColumn", "TwoPageLeft", "TwoColumnLeft", "TwoPageRight", "TwoColumnRight"],
            "ja": ["単一ページ表示", "スクロールを有効にする", "見開きページ表示", "見開きページでスクロール", "見開きページ\n(表紙は単独表示)", "見開きページでスクロール\n(表紙は単独表示)"],
        }
        self.pagelayout_value = ["SinglePage", "OneColumn", "TwoPageLeft", "TwoColumnLeft", "TwoPageRight", "TwoColumnRight"]
        self.pagelayout_radio = []
        for i, pagelayout in enumerate(self.pagelayout_text[self.language]):
            pagelayout_radio = ttk.Radiobutton(
                self.pagelayout_labelframe, text=pagelayout, variable=self.pagelayout_var, value=self.pagelayout_value[i]
            )
            self.pagelayout_radio.append(pagelayout_radio)
            pagelayout_radio.grid(row=i+1, column=1, sticky="nsew", padx=5, pady=1)

        # Create a LabelFrame for the direction options
        self.direction_label_text = {"en": "Direction:", "ja": "綴じ方向:"}
        self.direction_labelframe = ttk.LabelFrame(direction_frame, text=self.direction_label_text[self.language], padding=5)
        self.direction_labelframe.grid(row=0, column=2, rowspan=10, sticky="nsew", padx=5, pady=1)

        self.direction_var = tk.StringVar(value="R2L")
        self.direction_text = {
            "en": ["L2R", "R2L"],
            "ja": ["左綴じ", "右綴じ"],
        }
        self.direction_value = ["L2R", "R2L"]
        self.direction_radio = []
        for i, direction in enumerate(self.direction_text[self.language]):
            direction_radio = ttk.Radiobutton(
                self.direction_labelframe, text=direction, variable=self.direction_var, value=self.direction_value[i])
            self.direction_radio.append(direction_radio)
            direction_radio.grid(row=i+1, column=2, sticky="nsew", padx=5, pady=1)

        # Add language toggle button
        self.language_button_text = {"en": "日本語表示に切替", "ja": "Display in English"}
        self.language_button = ttk.Button(button_frame, text=self.language_button_text[self.language], command=self.toggle_language)
        self.language_button.pack(side="left", padx=15, pady=5)

        # Add quit button
        self.quit_button_text = {"en": "Quit", "ja": "終了"}
        self.quit_button = ttk.Button(button_frame, text=self.quit_button_text[self.language], command=sys.exit)
        self.quit_button.pack(side="left", padx=15, pady=5)

        # Add convert button
        self.convert_button_text = {"en": "Convert", "ja": "変換"}
        self.convert_button = ttk.Button(button_frame, text=self.convert_button_text[self.language], command=self.run_convert)
        self.convert_button.pack(side="right", padx=15, pady=5)

        # Add padding to all widgets
        for child in master.winfo_children():
            child.grid_configure(padx=10, pady=5)

        # Set minimum size of the window
        master.minsize(400, 250)

        # Center the window
        master.eval('tk::PlaceWindow . center')

    def toggle_language(self):
        # Toggle the language between english and japanese
        if self.language == "en":
            self.language = "ja"
            self.language_button.configure(text="Display in English")
        else:
            self.language = "en"
            self.language_button.configure(text="日本語表示に切替")
        self.update_language()

    def update_language(self):
        self.input_file_button.configure(text=self.input_file_button_text[self.language])
        self.input_directory_button.configure(text=self.input_directory_button_text[self.language])
        self.input_label.configure(text=self.input_label_text[self.language])
        self.output_label.configure(text=self.output_label_text[self.language])
        self.output_browse_button.configure(text=self.output_browse_button_text[self.language])
        self.auto_output_button.configure(text=self.auto_output_button_text[self.language])
        self.conversion_labelframe.configure(text=self.conversion_label_text[self.language])
        for i, conversion in enumerate(self.conversion_text[self.language]):
            self.conversion_radio[i].configure(text=conversion)
        self.conversion_var.set(self.conversion_var.get())
        self.pagelayout_labelframe.configure(text=self.pagelayout_label_text[self.language])
        for i, pagelayout in enumerate(self.pagelayout_text[self.language]):
            self.pagelayout_radio[i].configure(text=pagelayout)
        self.pagelayout_var.set(self.pagelayout_var.get())
        self.direction_labelframe.configure(text=self.direction_label_text[self.language])
        for i, direction in enumerate(self.direction_text[self.language]):
            self.direction_radio[i].configure(text=direction)
        self.direction_var.set(self.direction_var.get())
        self.language_button.configure(text=self.language_button_text[self.language])
        self.convert_button.configure(text=self.convert_button_text[self.language])
        self.quit_button.configure(text=self.quit_button_text[self.language])

    def set_output_path(self, path):
        self.output_path = path
        self.output_entry.delete(0, tk.END)
        self.output_entry.insert(0, path)

    def browse_output_path(self):
        path = filedialog.asksaveasfilename(defaultextension=".pdf")
        if path:
            self.set_output_path(path)

    def browse_input_file(self):
        filetypes = (("zip files", "*.zip"), ("cbz files", "*.cbz"), ("rar files", "*.rar"), ("cbr files", "*.cbr"), ("epub files", "*.epub"), ("all files", "*.*"))
        path = filedialog.askopenfilename(filetypes=filetypes)
        if path:
            self.input_path = path.replace('/', os.sep)
            self.set_output_path(os.path.splitext(path)[0] + ".pdf")
            self.input_entry.delete(0, tk.END)
            self.input_entry.insert(0, path)

    def browse_input_directory(self):
        path = filedialog.askdirectory()
        if path:
            self.input_path = path
            # set output path to the input directory with ".pdf" extension
            output_path = os.path.join(path, os.path.basename(path) + ".pdf").replace(os.sep, '/')
            self.set_output_path(output_path)
            self.input_entry.delete(0, tk.END)
            self.input_entry.insert(0, path)
    
    def auto_output_path(self):
        # Get input path
        input_path = self.input_entry.get()

        # Notify user if no input path is provided
        if input_path == "":
            messagebox.showerror("Error", "Please select an input path first.", parent=self.master)
            return

        # Get output path
        output_path = self.output_entry.get()

        # Determine the output path if it is not specified
        if not output_path:
            input_basename = os.path.basename(input_path)
            input_directory = os.path.dirname(input_path)
            if os.path.isdir(input_path):
                output_path = os.path.join(input_path, input_basename + ".pdf").replace(os.sep, '/')
            else:
                output_path = os.path.join(input_directory, os.path.splitext(input_basename)[0] + ".pdf").replace(os.sep, '/')
            self.output_entry.delete(0, tk.END)
            self.output_entry.insert(0, output_path)
        # Notify user if the output path is not a PDF file
        elif not output_path.endswith(".pdf"):
            if os.path.isdir(output_path):
                input_basename = os.path.splitext(os.path.basename(input_path))[0]
                output_path = os.path.join(output_path, input_basename + ".pdf").replace(os.sep, '/')
                self.output_entry.delete(0, tk.END)
                self.output_entry.insert(0, output_path)
            else:
                messagebox.showerror("Error", "Output path is not a PDF file", parent=self.master)
                return

        # Set the output path and filename
        self.output_path = output_path

    def run_convert(self):
        # Get input and output paths
        input_path = self.input_entry.get()
        output_path = self.output_entry.get()

        # Check if input path is empty
        if not input_path:
            messagebox.showerror("Error", "Please specify an input path.", parent=self.master)
            return

        # Determine the output path if it is not specified
        if not output_path:
            if os.path.isdir(input_path):
                input_dir, dir_name = os.path.split(input_path)
                output_path = os.path.join(input_dir, f"{dir_name}.pdf").replace(os.sep, '/')
            else:
                output_path = os.path.splitext(input_path)[0] + ".pdf"

        # Notify user if no output path is provided
        if not output_path:
            messagebox.showerror("Error", "Please specify an output path.", parent=self.master)
            return
        
        # Determine the conversion options
        convert_to_jpeg = self.conversion_var.get() == "jpeg"
        convert_to_grayscale = self.conversion_var.get() == "grayscale"

        # Call MangaPdfConverter with the appropriate arguments
        try:
            converter = MangaPdfConverter(input_path=input_path, output_path=output_path, pagelayout=self.pagelayout_var.get(), direction=self.direction_var.get())

            # Ask user if they want to convert
            confirm_text = {"en": "Are you sure you want to convert?", "ja": "変換処理を開始しますか？"}
            answer = messagebox.askyesno("Confirm Conversion", confirm_text[self.language], parent=self.master)

            if not answer:
                cancele_text = {"en": "Conversion canceled.", "ja": "変換処理を中止しました"}
                messagebox.showinfo("Conversion", cancele_text[self.language], parent=self.master)
                return
            
            # Create "Processing..." window
            processing_window = tk.Toplevel(self.master)
            # Get the position and size of the parent window
            main_x, main_y = self.master.winfo_x(), self.master.winfo_y()
            main_width, main_height = self.master.winfo_width(), self.master.winfo_height()
            # Calculate the position of the processing window
            sub_width, sub_height = 250, 30
            sub_x = main_x + (main_width - sub_width) // 2
            sub_y = main_y + (main_height - sub_height) // 2
            processing_window.geometry(f"{sub_width}x{sub_height}+{sub_x}+{sub_y}")
            # Set the title of the window
            processing_window.title("Manga PDF Converter")
            # Set the parent of the processing window
            processing_window.transient(self.master)
            # Add widgets to the frame
            processing_text = {"en": "Processing...", "ja": "変換処理中..."}
            processing_label = tk.Label(processing_window, text=processing_text[self.language])
            processing_label.pack()

            # Disable main window while process window is open
            #self.master.withdraw()

            # Make the processing window a modal dialog
            processing_window.lift()
            processing_window.focus_force()
            processing_window.wait_visibility()
            processing_window.grab_set()
            processing_label.update()

            if convert_to_jpeg:
                converter.set_convert_to_jpeg(True)
                converter.convert()
            elif convert_to_grayscale:
                converter.set_convert_to_grayscale(True)
                converter.convert()
            else:
                converter.convert()

            # Close process window when done
            processing_window.grab_release()
            processing_window.destroy()

            # Re-enable main window
            #self.master.deiconify()

            # Check if output file was created
            if os.path.exists(output_path) and os.path.getsize(output_path) > 0:
                complete_success_text = {"en": "Conversion complete!", "ja": "変換処理が完了しました！"}
                messagebox.showinfo("Success", complete_success_text[self.language], parent=self.master)
            else:
                complete_error_text = {"en": "Conversion failed", "ja": "変換処理に失敗しました"}
                messagebox.showerror("Error", complete_error_text[self.language], parent=self.master)

        except Exception as e:
            # Close process window when done
            processing_window.destroy()

            complete_error_text = {"en": "Conversion failed", "ja": "エラーで変換処理に失敗しました"}
            messagebox.showerror(title="Error", message=f"{complete_error_text[self.language]}\n{str(e)}", parent=self.master)

            # Re-enable main window
            self.master.deiconify()

def launch_gui():
    root = tk.Tk()
    MangaPdfConverterGUI(root)
    root.mainloop()

if __name__ == '__main__':
    launch_gui()


from setuptools import setup, find_packages

VERSION = "0.1.1"

INSTALL_REQUIRES = (
    "numpy",
    "img2pdf",
    "Pillow",
    "pikepdf",
    "rarfile",
    "beautifulsoup4"
)
CLASSIFIERS=[
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10',
    'Programming Language :: Python :: 3.11'
]

setup(
    name="manga2pdf",
    version=VERSION,
    author="mashu3",
    description="Convert manga/comic files(zip, epub, etc.) or directory containing image files (jpg, png, etc.) to PDF.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    keywords="manga comic pdf converter",
    url="https://github.com/mashu3/manga2pdf",
    license='MIT',
    package_dir={"": "src"},
    py_modules=["manga2pdf", "manga2pdf_gui"],
    packages = find_packages("src"),
    classifiers=CLASSIFIERS,
    install_requires=INSTALL_REQUIRES,
    entry_points={
        "console_scripts": [
            "manga2pdf=manga2pdf:main",
        ]
    }
)